import com.rinearn.graph2d.RinearnGraph2D;
import com.rinearn.graph2d.RinearnGraph2DOptionItem;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;


/**
 * 座標値データを連続的に送り続けて、
 * グラフをアニメーション表示させるサンプルコードです。
 */
public class Sample4 implements Runnable, WindowListener {
	
	Thread thread = null; // アニメーション用のスレッド
	RinearnGraph2D graph = null; // グラフ
	volatile boolean continuesLoop = true; // ループの継続/終了を制御する
	
	// mainメソッド(最初に実行されます)
	public static void main(String[] args) {
		new Sample4();
	}
	
	// 初期化・実行開始処理
	public Sample4() {
		
		// グラフを起動し、描画範囲の設定と自動調整機能の無効化を行う
		this.graph = new RinearnGraph2D();
		graph.setXRange(0.0, 10.0);
		graph.setYRange(-2.5, 2.5);
		this.graph.setXAutoRangingEnabled(false);
		this.graph.setYAutoRangingEnabled(false);
		
		// データ更新と描画処理の関係を非同期にする(リアルタイムアニメーション用)
		//（※ 連番で画像出力する用途などでは行わない方がコマ落ちや欠けを防げる）
		this.graph.setAsynchronousPlottingEnabled(true);
		
		// グラフを閉じたら独自終了処理を行うリスナーを登録、デフォルト処理は無効化
		this.graph.addWindowListener(this);
		this.graph.setAutoDisposingEnabled(false);
		
		// アニメーションスレッドを生成して実行開始
		this.thread = new Thread(this);
		this.thread.start();
	}
	
	// アニメーションスレッドの処理
	@Override
	public void run() {
		int n = 101;
		double[] x = new double[n];
		double[] y = new double[n];
		
		// アニメーションループ（loopStateがtrueの間継続）
		for(int frame=0; this.continuesLoop; frame++) {
			double t = frame * 0.05; // 時刻変数
			
			// 座標値データの更新
			for(int i=0; i<n; i++) {
				x[i] = i * 0.1;
				y[i] = Math.sin(3.0*x[i]+1.5*t)+Math.cos(3.5*x[i]+t);
			}
			// 座標値データをグラフに転送(非同期)
			this.graph.setData(x, y);
            
			// 50ミリ秒だけ停止(時間は適時調整)
			try {
				Thread.sleep(30);
			} catch(InterruptedException e) {
				// 割り込み例外の処理
			}
		}
		
		// アニメーションループが終了したらグラフを破棄
		this.graph.dispose();
		
		// スレッド処理終端: 他にスレッド・リソースが残っていなければ自然に実行終了
	}
	
	
	// ※ 以下のようなイベント処理の実装が面倒な場合は、多少強引でよければ、
	// this.graph.setAutoExittingEnabled(true); により、グラフを閉じたら
	// アプリケーションの実行を即終了するよう設定可能です（即席の場合向けです）。
	
	// グラフのウィンドウが閉じられた際に行うイベント処理
	@Override
	public void windowClosing(WindowEvent e) {
		// アニメーションループを脱出させ、スレッドを終了させる（結果、実行も終了）
		this.continuesLoop = false;
	}
	
	// その他のウィンドウイベント処理（ここでは何もしない）
	@Override
	public void windowDeactivated(WindowEvent e) { }
	@Override
	public void windowActivated(WindowEvent e) { }
	@Override
	public void windowDeiconified(WindowEvent e) { }
	@Override
	public void windowIconified(WindowEvent e) { }
	@Override
	public void windowOpened(WindowEvent e) { }
	@Override
	public void windowClosed(WindowEvent e) { }
}
